@@ -22,11 +22,9 @@
 #ifndef SEAHORSEGKR_H_
 #define SEAHORSEGKR_H_
 
-#include "common/seahorse-registry.h"
-
 #define SEAHORSE_GKR_STR                     "gnome-keyring"
 #define SEAHORSE_GKR                         (g_quark_from_static_string (SEAHORSE_GKR_STR))
 
-extern const SeahorseRegisterType SEAHORSE_GKR_REGISTRY[];
+void             seahorse_gkr_module_init        (void);
 
 #endif /*SEAHORSEGKR_H_*/
