@@ -43,11 +43,14 @@
 #include <unistd.h>
 #include <time.h>
 #include <stdio.h>
+#include <stdlib.h>
 #include <unistd.h>
 #include <string.h>
 #include <errno.h>
 #include <glib/gi18n.h>
 
+#include <sys/types.h>
+
 static const gchar *bad_filename_chars = "/\\<>|";
 
 /**
@@ -77,7 +80,7 @@ seahorse_util_show_error (GtkWidget *parent, const gchar *heading, const gchar *
 		} else {
 			if (!GTK_IS_WINDOW (parent)) 
 				parent = gtk_widget_get_toplevel (parent);
-			if (!GTK_IS_WINDOW (parent) && GTK_WIDGET_TOPLEVEL (parent))
+			if (!GTK_IS_WINDOW (parent) && gtk_widget_is_toplevel (parent))
 				parent = NULL;
 		}
 	}
@@ -151,7 +154,7 @@ seahorse_util_prompt_delete (const gchar *text, GtkWidget *parent)
 		} else {
 			if (!GTK_IS_WINDOW (parent)) 
 				parent = gtk_widget_get_toplevel (parent);
-			if (!GTK_IS_WINDOW (parent) && GTK_WIDGET_TOPLEVEL (parent))
+			if (!GTK_IS_WINDOW (parent) && gtk_widget_is_toplevel (parent))
 				parent = NULL;
 		}
 	}
@@ -307,8 +310,11 @@ seahorse_util_read_to_memory (GInputStream *input, guint *len)
 	string = g_string_new ("");
 	buffer = g_new (gchar, size);
     
-	while (g_input_stream_read_all (input, buffer, size, &nread, NULL, NULL) && nread == size)
+	while (g_input_stream_read_all (input, buffer, size, &nread, NULL, NULL)) {
 		string = g_string_append_len (string, buffer, nread);
+		if (nread != size)
+			break;
+	}
 
 	if (len)
 		*len = string->len;
@@ -427,7 +433,7 @@ seahorse_util_memory_output_length (GMemoryOutputStream *output)
 }
 
 /** 
- * seahorse_util_printf_fd:
+ * seahorse_util_print_fd:
  * @fd: The file descriptor to write to
  * @s:  The data to write
  *
@@ -456,14 +462,14 @@ seahorse_util_print_fd (int fd, const char* s)
             l -= r;
         }
     }
-    
+
     return TRUE;
 }
 
 /** 
  * seahorse_util_printf_fd:
  * @fd: The file descriptor to write to
- * @data: The printf format of the data to write
+ * @fmt: The printf format of the data to write
  * @...: The parameters to insert
  *
  * Returns: TRUE on success, FALSE on error
@@ -1577,6 +1583,7 @@ seahorse_util_determine_popup_menu_position (GtkMenu *menu, int *x, int *y,
 {
         GtkWidget      *widget;
         GtkRequisition  requisition;
+        GtkAllocation   allocation;
         gint            menu_xpos;
         gint            menu_ypos;
 
@@ -1584,17 +1591,18 @@ seahorse_util_determine_popup_menu_position (GtkMenu *menu, int *x, int *y,
 
         gtk_widget_size_request (GTK_WIDGET (menu), &requisition);
 
-        gdk_window_get_origin (widget->window, &menu_xpos, &menu_ypos);
+        gdk_window_get_origin (gtk_widget_get_window (widget), &menu_xpos, &menu_ypos);
 
-        menu_xpos += widget->allocation.x;
-        menu_ypos += widget->allocation.y;
+        gtk_widget_get_allocation (widget, &allocation);
+        menu_xpos += allocation.x;
+        menu_ypos += allocation.y;
 
 
         if (menu_ypos > gdk_screen_get_height (gtk_widget_get_screen (widget)) / 2)
                 menu_ypos -= requisition.height;
         else
-                menu_ypos += widget->allocation.height;
-           
+                menu_ypos += allocation.height;
+
         *x = menu_xpos;
         *y = menu_ypos;
         *push_in = TRUE;
