@@ -29,14 +29,16 @@
 
 #include "seahorse-ldap-source.h"
 
-#include "seahorse-gpgmex.h"
 #include "seahorse-pgp-key.h"
+#include "seahorse-pgp-subkey.h"
+#include "seahorse-pgp-uid.h"
 
 #include "seahorse-operation.h"
 #include "seahorse-servers.h"
 #include "seahorse-util.h"
 
 #include "common/seahorse-registry.h"
+#include "common/seahorse-object-list.h"
 
 #include <ldap.h>
 
@@ -255,29 +257,30 @@ get_date_attribute (LDAP* ld, LDAPMessage *res, const char *attribute)
     return d;         
 }
 
-static gpgme_pubkey_algo_t
+static const gchar*
 get_algo_attribute (LDAP* ld, LDAPMessage *res, const char *attribute)
 {
-    gpgme_pubkey_algo_t a = 0;
-    gchar **vals;
+	const gchar *a = NULL;
+	gchar **vals;
     
-    vals = get_ldap_values (ld, res, attribute);
-    if (!vals)
-        return 0;
+	vals = get_ldap_values (ld, res, attribute);
+	if (!vals)
+		return 0;
     
-    if (vals[0]) {
-        if (g_ascii_strcasecmp (vals[0], "DH/DSS") == 0 || 
-            g_ascii_strcasecmp (vals[0], "Elg") == 0 ||
-            g_ascii_strcasecmp (vals[0], "Elgamal") == 0)
-            a = GPGME_PK_ELG;
-        if (g_ascii_strcasecmp (vals[0], "RSA") == 0)
-            a = GPGME_PK_RSA;
-        if (g_ascii_strcasecmp (vals[0], "DSA") == 0)
-            a = GPGME_PK_DSA;     
-    }
+	if (vals[0]) {
+		if (g_ascii_strcasecmp (vals[0], "DH/DSS") == 0 || 
+		    g_ascii_strcasecmp (vals[0], "Elg") == 0 ||
+		    g_ascii_strcasecmp (vals[0], "Elgamal") == 0 ||
+		    g_ascii_strcasecmp (vals[0], "DSS/DH") == 0)
+			a = "Elgamal";
+		if (g_ascii_strcasecmp (vals[0], "RSA") == 0)
+			a = "RSA";
+		if (g_ascii_strcasecmp (vals[0], "DSA") == 0)
+			a = "DSA";     
+	}
     
-    g_strfreev (vals);
-    return a;
+	g_strfreev (vals);
+	return a;
 }
 
 /* 
@@ -748,67 +751,99 @@ static const char *kPGPAttributes[] = {
 };
 
 static void
-add_key (SeahorseLDAPSource *ssrc, gpgme_key_t key)
+add_key (SeahorseLDAPSource *ssrc, SeahorsePgpKey *key)
 {
-    SeahorseObject *prev;
-    SeahorsePGPKey *pkey;
-    GQuark keyid;
-       
-    keyid = seahorse_pgp_key_get_cannonical_id (seahorse_pgp_key_get_id (key, 0));
-    prev = seahorse_context_get_object (SCTX_APP (), SEAHORSE_SOURCE (ssrc), keyid);
+	SeahorseObject *prev;
+	GQuark keyid;
+
+	keyid = seahorse_pgp_key_canonize_id (seahorse_pgp_key_get_keyid (key));
+	prev = seahorse_context_get_object (SCTX_APP (), SEAHORSE_SOURCE (ssrc), keyid);
     
-    /* TODO: This function needs reworking after we get more key types */
-    if (prev != NULL) {
-        g_return_if_fail (SEAHORSE_IS_PGP_KEY (prev));
-        gpgmex_combine_keys (SEAHORSE_PGP_KEY (prev)->pubkey, key);
-        seahorse_object_fire_changed (prev, SKEY_CHANGE_UIDS);
-        return;
-    }
+	if (prev != NULL) {
+		g_return_if_fail (SEAHORSE_IS_PGP_KEY (prev));
+		seahorse_pgp_key_set_uids (SEAHORSE_PGP_KEY (prev), seahorse_pgp_key_get_uids (key));
+		seahorse_pgp_key_set_subkeys (SEAHORSE_PGP_KEY (prev), seahorse_pgp_key_get_subkeys (key));
+		return;
+	}
 
-    /* A public key */
-    pkey = seahorse_pgp_key_new (SEAHORSE_SOURCE (ssrc), key, NULL);
-
-    /* Add to context */ 
-    seahorse_context_add_object (SCTX_APP (), SEAHORSE_OBJECT (pkey));
+	/* Add to context */ 
+	seahorse_object_set_source (SEAHORSE_OBJECT (key), SEAHORSE_SOURCE (ssrc));
+	seahorse_context_add_object (SCTX_APP (), SEAHORSE_OBJECT (key));
 }
 
 /* Add a key to the key source from an LDAP entry */
 static void
 parse_key_from_ldap_entry (SeahorseLDAPOperation *lop, LDAPMessage *res)
 {
-    gpgme_pubkey_algo_t algo;
-    long int timestamp;
-    long int expires;
-    gpgme_key_t key;
-    gchar *fpr;
-    gchar *uid;
-    guint flags = 0;
-    int length;
+	const gchar *algo;
+	long int timestamp;
+	long int expires;
+        gchar *fpr, *fingerprint;
+        gchar *uidstr;
+        gboolean revoked;
+        gboolean disabled;
+        int length;
         
-    g_assert (SEAHORSE_IS_LDAP_OPERATION (lop));
-    g_return_if_fail (res && ldap_msgtype (res) == LDAP_RES_SEARCH_ENTRY);  
+        g_assert (SEAHORSE_IS_LDAP_OPERATION (lop));
+        g_return_if_fail (res && ldap_msgtype (res) == LDAP_RES_SEARCH_ENTRY);  
     
-    fpr = get_string_attribute (lop->ldap, res, "pgpcertid");
-    uid = get_string_attribute (lop->ldap, res, "pgpuserid");
-    flags |= (get_boolean_attribute (lop->ldap, res, "pgprevoked") ? GPGMEX_KEY_REVOKED : 0);
-    flags |= (get_boolean_attribute (lop->ldap, res, "pgpdisabled") ? GPGMEX_KEY_DISABLED : 0);
-    timestamp = get_date_attribute (lop->ldap, res, "pgpkeycreatetime");
-    expires = get_date_attribute (lop->ldap, res, "pgpkeyexpiretime");
-    algo = get_algo_attribute (lop->ldap, res, "pgpkeytype");
-    length = get_int_attribute (lop->ldap, res, "pgpkeysize");
+        fpr = get_string_attribute (lop->ldap, res, "pgpcertid");
+        uidstr = get_string_attribute (lop->ldap, res, "pgpuserid");
+        revoked = get_boolean_attribute (lop->ldap, res, "pgprevoked");
+        disabled = get_boolean_attribute (lop->ldap, res, "pgpdisabled");
+        timestamp = get_date_attribute (lop->ldap, res, "pgpkeycreatetime");
+        expires = get_date_attribute (lop->ldap, res, "pgpkeyexpiretime");
+        algo = get_algo_attribute (lop->ldap, res, "pgpkeytype");
+        length = get_int_attribute (lop->ldap, res, "pgpkeysize");
     
-    if (fpr && uid) {
-        key = gpgmex_key_alloc ();
-        gpgmex_key_add_subkey (key, fpr, flags, timestamp, 
-                               expires, length, algo);
-        gpgmex_key_add_uid (key, uid, flags);
+        if (fpr && uidstr) {
+
+        	SeahorsePgpSubkey *subkey;
+        	SeahorsePgpKey *key;
+        	SeahorsePgpUid *uid;
+                GList *list;
+                guint flags;
+
+        	/* Build up a subkey */
+        	subkey = seahorse_pgp_subkey_new ();
+        	seahorse_pgp_subkey_set_keyid (subkey, fpr);
+        	fingerprint = seahorse_pgp_subkey_calc_fingerprint (fpr);
+        	seahorse_pgp_subkey_set_fingerprint (subkey, fingerprint);
+        	g_free (fingerprint);
+        	seahorse_pgp_subkey_set_created (subkey, timestamp);
+        	seahorse_pgp_subkey_set_expires (subkey, expires);
+        	seahorse_pgp_subkey_set_algorithm (subkey, algo);
+        	seahorse_pgp_subkey_set_length (subkey, length);
+
+        	flags = SEAHORSE_FLAG_EXPORTABLE;
+        	if (revoked)
+        		flags |= SEAHORSE_FLAG_REVOKED;
+        	if (disabled)
+        		flags |= SEAHORSE_FLAG_DISABLED;
+       		seahorse_pgp_subkey_set_flags (subkey, flags);
+
+        	/* Build up a uid */
+        	uid = seahorse_pgp_uid_new (uidstr);
+        	if (revoked)
+        		seahorse_pgp_uid_set_validity (uid, SEAHORSE_VALIDITY_REVOKED);
+        	
+        	/* Now build them into a key */
+        	key = seahorse_pgp_key_new ();
+        	list = g_list_prepend (NULL, uid);
+        	seahorse_pgp_key_set_uids (key, list);
+        	seahorse_object_list_free (list);
+        	list = g_list_prepend (NULL, subkey);
+        	seahorse_pgp_key_set_subkeys (key, list);
+        	seahorse_object_list_free (list);
+        	g_object_set (key, "location", SEAHORSE_LOCATION_REMOTE, 
+        	              "flags", flags, NULL);
         
-        add_key (lop->lsrc, key);
-        gpgmex_key_unref (key);
-    }
+        	add_key (lop->lsrc, key);
+        	g_object_unref (key);
+        }
     
-    g_free (fpr);
-    g_free (uid);
+        g_free (fpr);
+        g_free (uidstr);
 }
 
 /* Got a search result */
@@ -923,6 +958,7 @@ start_search_operation (SeahorseLDAPSource *lsrc, const gchar *pattern)
     return lop;
 }
 
+#if 0 /* UNUSED */
 /* Initiate a search operation by fingerprint */
 static SeahorseLDAPOperation *
 start_search_operation_fpr (SeahorseLDAPSource *lsrc, const gchar *fpr)
@@ -949,6 +985,7 @@ start_search_operation_fpr (SeahorseLDAPSource *lsrc, const gchar *fpr)
     
     return lop;
 }
+#endif /* UNUSED */
 
 /* -----------------------------------------------------------------------------
  *  GET OPERATION 
@@ -1251,12 +1288,14 @@ start_send_operation_multiple (SeahorseLDAPSource *lsrc, GSList *keys)
  
 enum {
     PROP_0,
-    PROP_KEY_TYPE,
-    PROP_KEY_DESC
+    PROP_SOURCE_TAG,
+    PROP_SOURCE_LOCATION
 };
 
-G_DEFINE_TYPE (SeahorseLDAPSource, seahorse_ldap_source, SEAHORSE_TYPE_SERVER_SOURCE);
+static void seahorse_source_iface (SeahorseSourceIface *iface);
 
+G_DEFINE_TYPE_EXTENDED (SeahorseLDAPSource, seahorse_ldap_source, SEAHORSE_TYPE_SERVER_SOURCE, 0,
+                        G_IMPLEMENT_INTERFACE (SEAHORSE_TYPE_SOURCE, seahorse_source_iface));
 
 static void 
 seahorse_ldap_source_init (SeahorseLDAPSource *lsrc)
@@ -1269,40 +1308,20 @@ seahorse_ldap_source_get_property (GObject *object, guint prop_id, GValue *value
                                    GParamSpec *pspec)
 {
     switch (prop_id) {
-    case PROP_KEY_TYPE:
+    case PROP_SOURCE_TAG:
         g_value_set_uint (value, SEAHORSE_PGP);
         break;
-    case PROP_KEY_DESC:
-        g_value_set_string (value, _("PGP Key"));
+    case PROP_SOURCE_LOCATION:
+        g_value_set_enum (value, SEAHORSE_LOCATION_REMOTE);
         break;
     };        
 }
 
-static SeahorseOperation*
-seahorse_ldap_source_load (SeahorseSource *src, GQuark keyid)
+static void 
+seahorse_ldap_source_set_property (GObject *object, guint prop_id, const GValue *value,
+                                   GParamSpec *pspec)
 {
-    SeahorseOperation *op;
-    SeahorseLDAPOperation *lop = NULL;
 
-    g_assert (SEAHORSE_IS_SOURCE (src));
-    g_assert (SEAHORSE_IS_LDAP_SOURCE (src));
-
-    op = SEAHORSE_SOURCE_CLASS (seahorse_ldap_source_parent_class)->load (src, keyid);
-    if (op != NULL)
-        return op;
-    
-    /* No way to find new or all keys */
-    if (!keyid) 
-        return seahorse_operation_new_complete (NULL);
-
-    lop = start_search_operation_fpr (SEAHORSE_LDAP_SOURCE (src), 
-                                      seahorse_key_get_rawid (keyid));
-    g_return_val_if_fail (lop != NULL, NULL);
-
-    seahorse_server_source_take_operation (SEAHORSE_SERVER_SOURCE (src),
-                                           SEAHORSE_OPERATION (lop));
-    g_object_ref (lop);
-    return SEAHORSE_OPERATION (lop);
 }
 
 static SeahorseOperation*
@@ -1376,7 +1395,7 @@ seahorse_ldap_source_export_raw (SeahorseSource *sksrc, GSList *keyids,
     
     for (l = keyids; l; l = g_slist_next (l)) 
         fingerprints = g_slist_prepend (fingerprints, 
-            g_strdup (seahorse_key_get_rawid (GPOINTER_TO_UINT (l->data))));
+            g_strdup (seahorse_pgp_key_calc_rawid (GPOINTER_TO_UINT (l->data))));
     fingerprints = g_slist_reverse (fingerprints);
 
     lop = start_get_operation_multiple (lsrc, fingerprints, output);
@@ -1390,32 +1409,29 @@ static void
 seahorse_ldap_source_class_init (SeahorseLDAPSourceClass *klass)
 {
 	GObjectClass *gobject_class;
-	SeahorseSourceClass *key_class;
    
 	gobject_class = G_OBJECT_CLASS (klass);
 	gobject_class->get_property = seahorse_ldap_source_get_property;
-   
-	key_class = SEAHORSE_SOURCE_CLASS (klass);
-	key_class->canonize_id = seahorse_pgp_key_get_cannonical_id;
-	key_class->load = seahorse_ldap_source_load;
-	key_class->search = seahorse_ldap_source_search;
-	key_class->import = seahorse_ldap_source_import;
-	key_class->export_raw = seahorse_ldap_source_export_raw;
+	gobject_class->set_property = seahorse_ldap_source_set_property;
 
 	seahorse_ldap_source_parent_class = g_type_class_peek_parent (klass);
     
-	g_object_class_install_property (gobject_class, PROP_KEY_TYPE,
-	        g_param_spec_uint ("key-type", "Key Type", "Key type that originates from this key source.", 
-	                           0, G_MAXUINT, SKEY_UNKNOWN, G_PARAM_READABLE));
-
-	g_object_class_install_property (gobject_class, PROP_KEY_DESC,
-	        g_param_spec_string ("key-desc", "Key Desc", "Description for keys that originate here.",
-	                             NULL, G_PARAM_READABLE));
+	g_object_class_override_property (gobject_class, PROP_SOURCE_TAG, "source-tag");
+	g_object_class_override_property (gobject_class, PROP_SOURCE_LOCATION, "source-location");
 	    
-	seahorse_registry_register_type (NULL, SEAHORSE_TYPE_LDAP_SOURCE, "key-source", "remote", SEAHORSE_PGP_STR, NULL);
+	seahorse_registry_register_type (NULL, SEAHORSE_TYPE_LDAP_SOURCE, "source", "remote", SEAHORSE_PGP_STR, NULL);
 	seahorse_servers_register_type ("ldap", _("LDAP Key Server"), seahorse_ldap_is_valid_uri);
+
+	seahorse_registry_register_function (NULL, seahorse_pgp_key_canonize_id, "canonize", SEAHORSE_PGP_STR, NULL);
 }
 
+static void 
+seahorse_source_iface (SeahorseSourceIface *iface)
+{
+	iface->search = seahorse_ldap_source_search;
+	iface->import = seahorse_ldap_source_import;
+	iface->export_raw = seahorse_ldap_source_export_raw;
+}
 
 /**
  * seahorse_ldap_source_new
